/**
 * Copyright (C) 2018  The Software Heritage developers
 * See the AUTHORS file at the top-level directory of this distribution
 * License: GNU Affero General Public License version 3, or any later version
 * See top-level LICENSE file for more information
 */

.swh-visit-full {
    color: green;
    position: relative;
}

.swh-visit-full::before {
    content: '\f00c';
    font-family: 'FontAwesome';
    left: -20px;
    position: absolute;
    top: -2px;
}

.swh-visit-partial {
    color: #edc344;
    position: relative;
}

.swh-visit-partial::before {
    content: '\f071';
    font-family: 'FontAwesome';
    left: -20px;
    position: absolute;
    top: -2px;
}

.swh-visit-failed {
    color: #f00;
    position: relative;
}

.swh-visit-failed::before {
    content: '\f06a';
    font-family: 'FontAwesome';
    left: -20px;
    position: absolute;
    top: -2px;
}

.swh-visit-ongoing {
    color: #00f;
    position: relative;
}

.swh-visit-ongoing::before {
    content: '\f021';
    font-family: 'FontAwesome';
    left: -20px;
    position: absolute;
    top: -2px;
}

#swh-visits-calendar.calendar table td {
    width: 28px;
    height: 28px;
    padding: 0;
}

.d3-wrapper {
    position: relative;
    height: 0;
    width: 100%;
    padding: 0;

    /* padding-bottom will be overwritten by JavaScript later */
    padding-bottom: 100%;
}

.d3-wrapper > svg {
    position: absolute;
    height: 100%;
    width: 100%;
    left: 0;
    top: 0;
}

svg .grid line {
    stroke: lightgrey;
    stroke-opacity: 0.7;
    shape-rendering: crispEdges;
}

svg .grid path {
    stroke-width: 0;
}

div.d3-tooltip {
    position: absolute;
    text-align: center;
    width: auto;
    height: auto;
    padding: 2px;
    font: 12px sans-serif;
    background: white;
    border: 1px solid black;
    border-radius: 4px;
    pointer-events: none;
}

.swh-visits-list-column {
    float: left;
    padding: 10px;
}

.swh-visits-list-row {
    padding-left: 50px;
}

.swh-visits-list-row::after {
    content: "";
    display: table;
    clear: both;
}
