.swh-diff-lines-info{background-color:rgba(0,0,255,.1)!important}.swh-diff-added-line{background-color:rgba(0,255,0,.1)!important}.swh-diff-removed-line{background-color:rgba(255,0,0,.1)!important}span.no-nl-marker{position:relative;color:#cb2431;vertical-align:middle}span.no-nl-marker svg{vertical-align:text-bottom}span.no-nl-marker svg path{fill:currentColor}tr.swh-revision-log-entry td a{text-decoration:none}.swh-revision-log-entry-id{min-width:110px;max-width:110px;width:110px}.swh-revision-log-entry-author{min-width:160px;max-width:160px;width:160px}.swh-revision-log-entry-commit-date,.swh-revision-log-entry-date{min-width:200px;max-width:200px;width:200px}