{% extends "layout.html" %}

{% comment %}
Copyright (C) 2018  The Software Heritage developers
See the AUTHORS file at the top-level directory of this distribution
License: GNU Affero General Public License version 3, or any later version
See top-level LICENSE file for more information
{% endcomment %}

{% load swh_templatetags %}
{% load render_bundle from webpack_loader %}

{% block header %}
{{ block.super }}
{% render_bundle 'admin' %}
{% endblock %}

{% block title %} Deposit administration {% endblock %}

{% block navbar-content %}
<h4>Deposit administration</h4>
{% endblock %}

{% block content %}
  <p>
    The table below displays the whole list of software deposits into the archive
    submitted through <a href="https://hal.archives-ouvertes.fr/">HAL</a>.
  </p>
  <table id="swh-admin-deposit-list" class="table swh-table swh-table-striped" width="100%">
    <thead>
      <tr>
        <th>deposit id</th>
        <th>external id</th>
        <th>reception date</th>
        <th>status</th>
        <th>status detail</th>
        <th>swh id</th>
      </tr>
    </thead>
  </table>
  <p id="swh-admin-deposit-list-error"></p>
  <script>
    swh.webapp.initPage('deposit-admin');
    swh.admin.initDepositAdmin();
  </script>
{% endblock content %}